@@ -1,18 +1,21 @@
 use crate::ctype::{isalpha, isdigit, ispunct, isspace};
 use crate::nodes::{
-    AstNode, ListDelimType, ListType, NodeCodeBlock, NodeHeading, NodeHtmlBlock, NodeLink,
-    NodeMath, NodeTable, NodeValue, NodeWikiLink,
+    AstNode, ListDelimType, ListType, NodeAlert, NodeCodeBlock, NodeHeading, NodeHtmlBlock,
+    NodeLink, NodeMath, NodeTable, NodeValue, NodeWikiLink,
 };
 use crate::nodes::{NodeList, TableAlignment};
 #[cfg(feature = "shortcodes")]
 use crate::parser::shortcodes::NodeShortCode;
-use crate::parser::Options;
+use crate::parser::{Options, WikiLinksMode};
 use crate::scanners;
 use crate::strings::trim_start_match;
 use crate::{nodes, Plugins};
+pub use typed_arena::Arena;
 
 use std::cmp::max;
+use std::io::BufWriter;
 use std::io::{self, Write};
+use std::str;
 
 /// Formats an AST as CommonMark, modified by the given options.
 pub fn format_document<'a>(
@@ -40,10 +43,14 @@ pub fn format_document_with_plugins<'a>(
 ) -> io::Result<()> {
     let mut f = CommonMarkFormatter::new(root, options);
     f.format(root);
-    if !f.v.is_empty() && f.v[f.v.len() - 1] != b'\n' {
-        f.v.push(b'\n');
+    let mut result = f.v;
+    if !result.is_empty() && result[result.len() - 1] != b'\n' {
+        result.push(b'\n');
     }
-    output.write_all(&f.v)?;
+    if options.render.experimental_minimize_commonmark {
+        minimize_commonmark(&mut result, options);
+    }
+    output.write_all(&result)?;
     Ok(())
 }
 
@@ -366,6 +373,7 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
             NodeValue::HtmlInline(ref literal) => {
                 self.format_html_inline(literal.as_bytes(), entering)
             }
+            NodeValue::Raw(ref literal) => self.format_raw(literal.as_bytes(), entering),
             NodeValue::Strong => {
                 if parent_node.is_none()
                     || !matches!(parent_node.unwrap().data.borrow().value, NodeValue::Strong)
@@ -397,8 +405,10 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
             NodeValue::Math(ref math) => self.format_math(math, allow_wrap, entering),
             NodeValue::WikiLink(ref nl) => return self.format_wikilink(nl, entering),
             NodeValue::Underline => self.format_underline(),
+            NodeValue::Subscript => self.format_subscript(),
             NodeValue::SpoileredText => self.format_spoiler(),
             NodeValue::EscapedTag(ref net) => self.format_escaped_tag(net),
+            NodeValue::Alert(ref alert) => self.format_alert(alert, entering),
         };
         true
     }
@@ -465,24 +475,38 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
         let marker_width = if parent.list_type == ListType::Bullet {
             2
         } else {
-            let last_stack = self.ol_stack.last_mut().unwrap();
-            let list_number = *last_stack;
-            if entering {
-                *last_stack += 1;
-            }
+            let list_number = if let Some(last_stack) = self.ol_stack.last_mut() {
+                let list_number = *last_stack;
+                if entering {
+                    *last_stack += 1;
+                };
+                list_number
+            } else {
+                match node.data.borrow().value {
+                    NodeValue::Item(ref ni) => ni.start,
+                    NodeValue::TaskItem(_) => parent.start,
+                    _ => unreachable!(),
+                }
+            };
             let list_delim = parent.delimiter;
             write!(
                 listmarker,
-                "{}{}{}",
+                "{}{} ",
                 list_number,
                 if list_delim == ListDelimType::Paren {
                     ")"
                 } else {
                     "."
-                },
-                if list_number < 10 { "  " } else { " " }
+                }
             )
             .unwrap();
+            let mut current_len = listmarker.len();
+
+            while current_len < self.options.render.ol_width {
+                write!(listmarker, " ").unwrap();
+                current_len += 1;
+            }
+
             listmarker.len()
         };
 
@@ -498,7 +522,11 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
                 write!(self.prefix, " ").unwrap();
             }
         } else {
-            let new_len = self.prefix.len() - marker_width;
+            let new_len = if self.prefix.len() > marker_width {
+                self.prefix.len() - marker_width
+            } else {
+                0
+            };
             self.prefix.truncate(new_len);
             self.cr();
         }
@@ -627,9 +655,9 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
             {
                 self.cr();
             } else if self.options.render.hardbreaks {
-                self.output(&[b'\n'], allow_wrap, Escaping::Literal);
+                self.output(b"\n", allow_wrap, Escaping::Literal);
             } else {
-                self.output(&[b' '], allow_wrap, Escaping::Literal);
+                self.output(b" ", allow_wrap, Escaping::Literal);
             }
         }
     }
@@ -667,6 +695,12 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
         }
     }
 
+    fn format_raw(&mut self, literal: &[u8], entering: bool) {
+        if entering {
+            self.write_all(literal).unwrap();
+        }
+    }
+
     fn format_strong(&mut self) {
         write!(self, "**").unwrap();
     }
@@ -694,7 +728,7 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
     }
 
     fn format_strikethrough(&mut self) {
-        write!(self, "~").unwrap();
+        write!(self, "~~").unwrap();
     }
 
     fn format_superscript(&mut self) {
@@ -705,6 +739,10 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
         write!(self, "__").unwrap();
     }
 
+    fn format_subscript(&mut self) {
+        write!(self, "~").unwrap();
+    }
+
     fn format_spoiler(&mut self) {
         write!(self, "||").unwrap();
     }
@@ -738,12 +776,12 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
     fn format_wikilink(&mut self, nl: &NodeWikiLink, entering: bool) -> bool {
         if entering {
             write!(self, "[[").unwrap();
-            if self.options.extension.wikilinks_title_after_pipe {
+            if self.options.extension.wikilinks() == Some(WikiLinksMode::UrlFirst) {
                 self.output(nl.url.as_bytes(), false, Escaping::Url);
                 write!(self, "|").unwrap();
             }
         } else {
-            if self.options.extension.wikilinks_title_before_pipe {
+            if self.options.extension.wikilinks() == Some(WikiLinksMode::TitleFirst) {
                 write!(self, "|").unwrap();
                 self.output(nl.url.as_bytes(), false, Escaping::Url);
             }
@@ -760,7 +798,7 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
             write!(self, "](").unwrap();
             self.output(nl.url.as_bytes(), false, Escaping::Url);
             if !nl.title.is_empty() {
-                self.output(&[b' ', b'"'], allow_wrap, Escaping::Literal);
+                self.output(b" \"", allow_wrap, Escaping::Literal);
                 self.output(nl.title.as_bytes(), false, Escaping::Title);
                 write!(self, "\"").unwrap();
             }
@@ -874,6 +912,29 @@ impl<'a, 'o, 'c> CommonMarkFormatter<'a, 'o, 'c> {
             self.output(end_fence.as_bytes(), false, Escaping::Literal);
         }
     }
+
+    fn format_alert(&mut self, alert: &NodeAlert, entering: bool) {
+        if entering {
+            write!(
+                self,
+                "> [!{}]",
+                alert.alert_type.default_title().to_uppercase()
+            )
+            .unwrap();
+            if alert.title.is_some() {
+                let title = alert.title.as_ref().unwrap();
+                write!(self, " {}", title).unwrap();
+            }
+            writeln!(self).unwrap();
+            write!(self, "> ").unwrap();
+            self.begin_content = true;
+            write!(self.prefix, "> ").unwrap();
+        } else {
+            let new_len = self.prefix.len() - 2;
+            self.prefix.truncate(new_len);
+            self.blankline();
+        }
+    }
 }
 
 fn longest_char_sequence(literal: &[u8], ch: u8) -> usize {
@@ -947,3 +1008,33 @@ fn table_escape<'a>(node: &'a AstNode<'a>, c: u8) -> bool {
         _ => c == b'|',
     }
 }
+
+fn minimize_commonmark(text: &mut Vec<u8>, original_options: &Options) {
+    let mut options_without = original_options.clone();
+    options_without.render.experimental_minimize_commonmark = false;
+
+    let ixs: Vec<usize> = text
+        .iter()
+        .enumerate()
+        .filter_map(|(ix, &c)| if c == b'\\' { Some(ix) } else { None })
+        .collect();
+    let original = text.clone();
+
+    let mut adjust = 0;
+    for ix in ixs {
+        text.remove(ix - adjust);
+
+        let arena = Arena::new();
+        let root = crate::parse_document(&arena, str::from_utf8(text).unwrap(), &options_without);
+
+        let mut bw = BufWriter::new(Vec::new());
+        format_document(root, &options_without, &mut bw).unwrap();
+        let result = bw.into_inner().unwrap();
+
+        if original == result {
+            adjust += 1;
+        } else {
+            text.insert(ix - adjust, b'\\');
+        }
+    }
+}
