@@ -1,9 +1,8 @@
 //! The `comrak` binary.
 
 use comrak::{
-    adapters::SyntaxHighlighterAdapter, plugins::syntect::SyntectAdapter, Arena,
-    ExtensionOptionsBuilder, ListStyleType, Options, ParseOptionsBuilder, Plugins,
-    RenderOptionsBuilder,
+    adapters::SyntaxHighlighterAdapter, plugins::syntect::SyntectAdapter, Arena, ListStyleType,
+    Options, Plugins,
 };
 use std::boxed::Box;
 use std::env;
@@ -14,6 +13,7 @@ use std::path::PathBuf;
 use std::process;
 
 use clap::{Parser, ValueEnum};
+use comrak::{ExtensionOptions, ParseOptions, RenderOptions};
 
 const EXIT_SUCCESS: i32 = 0;
 const EXIT_PARSE_CONFIG: i32 = 2;
@@ -76,6 +76,10 @@ struct Cli {
     #[arg(long)]
     relaxed_autolinks: bool,
 
+    /// Output classes on tasklist elements so that they can be styled with CSS
+    #[arg(long)]
+    tasklist_classes: bool,
+
     /// Default value for fenced code block's info strings if none is given
     #[arg(long, value_name = "INFO")]
     default_info_string: Option<String>,
@@ -141,10 +145,6 @@ struct Cli {
     #[arg(long)]
     sourcepos: bool,
 
-    /// Include inline sourcepos in HTML output, which is known to have issues.
-    #[arg(long)]
-    experimental_inline_sourcepos: bool,
-
     /// Ignore setext headers
     #[arg(long)]
     ignore_setext: bool,
@@ -152,6 +152,10 @@ struct Cli {
     /// Ignore empty links
     #[arg(long)]
     ignore_empty_links: bool,
+
+    /// Minimize escapes in CommonMark output using a trial-and-error algorithm.
+    #[arg(long)]
+    experimental_minimize_commonmark: bool,
 }
 
 #[derive(Clone, Copy, Debug, ValueEnum)]
@@ -180,8 +184,10 @@ enum Extension {
     WikilinksTitleAfterPipe,
     WikilinksTitleBeforePipe,
     Underline,
+    Subscript,
     Spoiler,
     Greentext,
+    Alerts,
 }
 
 #[derive(Clone, Copy, Debug, ValueEnum)]
@@ -247,15 +253,14 @@ fn main() -> Result<(), Box<dyn Error>> {
 
     let exts = &cli.extensions;
 
-    let mut extension = ExtensionOptionsBuilder::default();
-    extension
+    let extension = ExtensionOptions::builder()
         .strikethrough(exts.contains(&Extension::Strikethrough) || cli.gfm)
         .tagfilter(exts.contains(&Extension::Tagfilter) || cli.gfm)
         .table(exts.contains(&Extension::Table) || cli.gfm)
         .autolink(exts.contains(&Extension::Autolink) || cli.gfm)
         .tasklist(exts.contains(&Extension::Tasklist) || cli.gfm)
         .superscript(exts.contains(&Extension::Superscript))
-        .header_ids(cli.header_ids)
+        .maybe_header_ids(cli.header_ids)
         .footnotes(exts.contains(&Extension::Footnotes))
         .description_lists(exts.contains(&Extension::DescriptionLists))
         .multiline_block_quotes(exts.contains(&Extension::MultilineBlockQuotes))
@@ -264,25 +269,25 @@ fn main() -> Result<(), Box<dyn Error>> {
         .wikilinks_title_after_pipe(exts.contains(&Extension::WikilinksTitleAfterPipe))
         .wikilinks_title_before_pipe(exts.contains(&Extension::WikilinksTitleBeforePipe))
         .underline(exts.contains(&Extension::Underline))
+        .subscript(exts.contains(&Extension::Subscript))
         .spoiler(exts.contains(&Extension::Spoiler))
         .greentext(exts.contains(&Extension::Greentext))
-        .front_matter_delimiter(cli.front_matter_delimiter);
+        .alerts(exts.contains(&Extension::Alerts))
+        .maybe_front_matter_delimiter(cli.front_matter_delimiter);
 
     #[cfg(feature = "shortcodes")]
-    {
-        extension.shortcodes(cli.gemojis);
-    }
+    let extension = extension.shortcodes(cli.gemojis);
 
-    let extension = extension.build()?;
+    let extension = extension.build();
 
-    let parse = ParseOptionsBuilder::default()
+    let parse = ParseOptions::builder()
         .smart(cli.smart)
-        .default_info_string(cli.default_info_string)
+        .maybe_default_info_string(cli.default_info_string)
         .relaxed_tasklist_matching(cli.relaxed_tasklist_character)
         .relaxed_autolinks(cli.relaxed_autolinks)
-        .build()?;
+        .build();
 
-    let render = RenderOptionsBuilder::default()
+    let render = RenderOptions::builder()
         .hardbreaks(cli.hardbreaks)
         .github_pre_lang(cli.github_pre_lang || cli.gfm)
         .full_info_string(cli.full_info_string)
@@ -291,12 +296,13 @@ fn main() -> Result<(), Box<dyn Error>> {
         .escape(cli.escape)
         .list_style(cli.list_style.into())
         .sourcepos(cli.sourcepos)
-        .experimental_inline_sourcepos(cli.experimental_inline_sourcepos)
+        .experimental_minimize_commonmark(cli.experimental_minimize_commonmark)
         .escaped_char_spans(cli.escaped_char_spans)
         .ignore_setext(cli.ignore_setext)
         .ignore_empty_links(cli.ignore_empty_links)
         .gfm_quirks(cli.gfm_quirks || cli.gfm)
-        .build()?;
+        .tasklist_classes(cli.tasklist_classes)
+        .build();
 
     let options = Options {
         extension,
@@ -358,7 +364,7 @@ fn main() -> Result<(), Box<dyn Error>> {
         formatter(root, &options, &mut bw, &plugins)?;
         bw.flush()?;
     } else if cli.inplace {
-        let output_filename = cli.files.unwrap().get(0).unwrap().clone();
+        let output_filename = cli.files.unwrap().first().unwrap().clone();
         let mut bw = BufWriter::new(fs::File::create(output_filename)?);
         formatter(root, &options, &mut bw, &plugins)?;
         bw.flush()?;
