@@ -1,14 +1,16 @@
+#![cfg(feature = "bon")]
+
 use std::sync::{Arc, Mutex};
 
 use parser::BrokenLinkReference;
 
+use super::*;
+use crate::nodes::AlertType;
 use crate::{
     adapters::{HeadingAdapter, HeadingMeta, SyntaxHighlighterAdapter},
     nodes::Sourcepos,
 };
 
-use super::*;
-
 #[test]
 fn exercise_full_api() {
     let arena = Arena::new();
@@ -32,14 +34,14 @@ fn exercise_full_api() {
     let _: &AstNode = parse_document(&arena, "document", &default_options);
 
     // Ensure the closure can modify its context.
-    let mut blr_ctx_0 = 0;
+    let blr_ctx_0 = Arc::new(Mutex::new(0));
     #[allow(deprecated)]
     let _: &AstNode = parse_document_with_broken_link_callback(
         &arena,
         "document",
         &Options::default(),
-        Some(&mut |blr: BrokenLinkReference| {
-            blr_ctx_0 += 1;
+        Arc::new(|blr: BrokenLinkReference| {
+            *blr_ctx_0.lock().unwrap() += 1;
             let _: &str = blr.normalized;
             let _: &str = blr.original;
             Some(ResolvedReference {
@@ -49,64 +51,64 @@ fn exercise_full_api() {
         }),
     );
 
-    let mut extension = ExtensionOptionsBuilder::default();
-    extension.strikethrough(false);
-    extension.tagfilter(false);
-    extension.table(false);
-    extension.autolink(false);
-    extension.tasklist(false);
-    extension.superscript(false);
-    extension.header_ids(Some("abc".to_string()));
-    extension.footnotes(false);
-    extension.description_lists(false);
-    extension.math_dollars(false);
-    extension.math_code(false);
-    extension.front_matter_delimiter(None);
-    extension.multiline_block_quotes(false);
-    extension.math_dollars(false);
-    extension.math_code(false);
+    let extension = ExtensionOptions::builder()
+        .strikethrough(false)
+        .tagfilter(false)
+        .table(false)
+        .autolink(false)
+        .tasklist(false)
+        .superscript(false)
+        .header_ids("abc".to_string())
+        .footnotes(false)
+        .description_lists(false)
+        .math_dollars(false)
+        .math_code(false)
+        .maybe_front_matter_delimiter(None)
+        .multiline_block_quotes(false);
+
     #[cfg(feature = "shortcodes")]
-    extension.shortcodes(true);
-    extension.wikilinks_title_after_pipe(true);
-    extension.wikilinks_title_before_pipe(true);
-    extension.underline(true);
-    extension.spoiler(true);
-    extension.greentext(true);
+    let extension = extension.shortcodes(true);
 
-    let mut parse = ParseOptionsBuilder::default();
-    parse.smart(false);
-    parse.default_info_string(Some("abc".to_string()));
-    parse.relaxed_tasklist_matching(false);
-    parse.relaxed_autolinks(false);
-    let mut blr_ctx_1 = 0;
-    parse.broken_link_callback(Some(Arc::new(Mutex::new(
-        &mut |blr: BrokenLinkReference| {
-            blr_ctx_1 += 1;
-            let _: &str = blr.normalized;
-            let _: &str = blr.original;
-            Some(ResolvedReference {
-                url: String::new(),
-                title: String::new(),
-            })
-        },
-    ))));
+    let _extension = extension
+        .wikilinks_title_after_pipe(true)
+        .wikilinks_title_before_pipe(true)
+        .underline(true)
+        .subscript(true)
+        .spoiler(true)
+        .greentext(true)
+        .alerts(true);
 
-    let mut render = RenderOptionsBuilder::default();
-    render.hardbreaks(false);
-    render.github_pre_lang(false);
-    render.full_info_string(false);
-    render.width(123456);
-    render.unsafe_(false);
-    render.escape(false);
-    render.list_style(ListStyleType::Dash);
-    render.sourcepos(false);
-    render.experimental_inline_sourcepos(false);
-    render.escaped_char_spans(false);
-    render.ignore_setext(true);
-    render.ignore_empty_links(true);
-    render.gfm_quirks(true);
-    render.prefer_fenced(true);
-    render.figure_with_caption(true);
+    let parse = ParseOptions::builder()
+        .smart(false)
+        .default_info_string("abc".to_string())
+        .relaxed_tasklist_matching(false)
+        .relaxed_autolinks(false);
+
+    let _parse = parse.broken_link_callback(Arc::new(|blr: BrokenLinkReference| {
+        *blr_ctx_0.lock().unwrap() += 1;
+        let _: &str = blr.normalized;
+        let _: &str = blr.original;
+        Some(ResolvedReference {
+            url: String::new(),
+            title: String::new(),
+        })
+    }));
+
+    let _render = RenderOptions::builder()
+        .hardbreaks(false)
+        .github_pre_lang(false)
+        .full_info_string(false)
+        .width(123456)
+        .unsafe_(false)
+        .escape(false)
+        .list_style(ListStyleType::Dash)
+        .sourcepos(false)
+        .escaped_char_spans(false)
+        .ignore_setext(true)
+        .ignore_empty_links(true)
+        .gfm_quirks(true)
+        .prefer_fenced(true)
+        .figure_with_caption(true);
 
     pub struct MockAdapter {}
     impl SyntaxHighlighterAdapter for MockAdapter {
@@ -153,12 +155,11 @@ fn exercise_full_api() {
 
     let mock_adapter = MockAdapter {};
 
-    let mut render_plugins = RenderPluginsBuilder::default();
-    render_plugins.codefence_syntax_highlighter(Some(&mock_adapter));
-    render_plugins.heading_adapter(Some(&mock_adapter));
+    let render_plugins = RenderPlugins::builder()
+        .codefence_syntax_highlighter(&mock_adapter)
+        .heading_adapter(&mock_adapter);
 
-    let mut plugins = PluginsBuilder::default();
-    plugins.render(render_plugins.build().unwrap());
+    let _plugins = Plugins::builder().render(render_plugins.build());
 
     let _: String = markdown_to_html("# Yes", &default_options);
 
@@ -240,6 +241,9 @@ fn exercise_full_api() {
         nodes::NodeValue::HtmlInline(html) => {
             let _: &String = html;
         }
+        nodes::NodeValue::Raw(text) => {
+            let _: &String = text;
+        }
         nodes::NodeValue::Emph => {}
         nodes::NodeValue::Strong => {}
         nodes::NodeValue::Strikethrough => {}
@@ -271,9 +275,15 @@ fn exercise_full_api() {
             let _: String = nl.url;
         }
         nodes::NodeValue::Underline => {}
+        nodes::NodeValue::Subscript => {}
         nodes::NodeValue::SpoileredText => {}
         nodes::NodeValue::EscapedTag(data) => {
             let _: &String = data;
         }
+        nodes::NodeValue::Alert(alert) => {
+            let _: AlertType = alert.alert_type;
+            let _: Option<String> = alert.title;
+            let _: bool = alert.multiline;
+        }
     }
 }
