@@ -1,6 +1,6 @@
+use crate::character_set::character_set;
 use crate::nodes::{AstNode, ListType, NodeCode, NodeMath, NodeTable, NodeValue};
 use crate::parser::{Options, Plugins};
-use once_cell::sync::Lazy;
 use std::cmp;
 use std::io::{self, Write};
 
@@ -48,17 +48,11 @@ impl<'o, 'c> XmlFormatter<'o, 'c> {
     }
 
     fn escape(&mut self, buffer: &[u8]) -> io::Result<()> {
-        static XML_SAFE: Lazy<[bool; 256]> = Lazy::new(|| {
-            let mut a = [true; 256];
-            for &c in b"&<>\"".iter() {
-                a[c as usize] = false;
-            }
-            a
-        });
+        const XML_UNSAFE: [bool; 256] = character_set!(b"&<>\"");
 
         let mut offset = 0;
         for (i, &byte) in buffer.iter().enumerate() {
-            if !XML_SAFE[byte as usize] {
+            if XML_UNSAFE[byte as usize] {
                 let esc: &[u8] = match byte {
                     b'"' => b"&quot;",
                     b'&' => b"&amp;",
@@ -95,7 +89,8 @@ impl<'o, 'c> XmlFormatter<'o, 'c> {
                         match node.data.borrow().value {
                             NodeValue::Text(ref literal)
                             | NodeValue::Code(NodeCode { ref literal, .. })
-                            | NodeValue::HtmlInline(ref literal) => {
+                            | NodeValue::HtmlInline(ref literal)
+                            | NodeValue::Raw(ref literal) => {
                                 self.escape(literal.as_bytes())?;
                             }
                             NodeValue::LineBreak | NodeValue::SoftBreak => {
@@ -154,22 +149,29 @@ impl<'o, 'c> XmlFormatter<'o, 'c> {
                 NodeValue::Text(ref literal)
                 | NodeValue::Code(NodeCode { ref literal, .. })
                 | NodeValue::HtmlBlock(NodeHtmlBlock { ref literal, .. })
-                | NodeValue::HtmlInline(ref literal) => {
+                | NodeValue::HtmlInline(ref literal)
+                | NodeValue::Raw(ref literal) => {
                     self.output.write_all(b" xml:space=\"preserve\">")?;
                     self.escape(literal.as_bytes())?;
                     write!(self.output, "</{}", ast.value.xml_node_name())?;
                     was_literal = true;
                 }
                 NodeValue::List(ref nl) => {
-                    if nl.list_type == ListType::Bullet {
-                        self.output.write_all(b" type=\"bullet\"")?;
-                    } else {
-                        write!(
-                            self.output,
-                            " type=\"ordered\" start=\"{}\" delim=\"{}\"",
-                            nl.start,
-                            nl.delimiter.xml_name()
-                        )?;
+                    match nl.list_type {
+                        ListType::Bullet => {
+                            self.output.write_all(b" type=\"bullet\"")?;
+                        }
+                        ListType::Ordered => {
+                            write!(
+                                self.output,
+                                " type=\"ordered\" start=\"{}\" delim=\"{}\"",
+                                nl.start,
+                                nl.delimiter.xml_name()
+                            )?;
+                        }
+                    }
+                    if nl.is_task_list {
+                        self.output.write_all(b" tasklist=\"true\"")?;
                     }
                     write!(self.output, " tight=\"{}\"", nl.tight)?;
                 }
@@ -279,10 +281,28 @@ impl<'o, 'c> XmlFormatter<'o, 'c> {
                     self.output.write_all(b"\"")?;
                 }
                 NodeValue::Underline => {}
+                NodeValue::Subscript => {}
                 NodeValue::SpoileredText => {}
                 NodeValue::EscapedTag(ref data) => {
                     self.output.write_all(data.as_bytes())?;
                 }
+                NodeValue::Alert(ref alert) => {
+                    self.output.write_all(b" type=\"")?;
+                    self.output
+                        .write_all(alert.alert_type.default_title().to_lowercase().as_bytes())?;
+                    self.output.write_all(b"\"")?;
+                    if alert.title.is_some() {
+                        let title = alert.title.as_ref().unwrap();
+
+                        self.output.write_all(b" title=\"")?;
+                        self.escape(title.as_bytes())?;
+                        self.output.write_all(b"\"")?;
+                    }
+
+                    if alert.multiline {
+                        self.output.write_all(b" multiline=\"true\"")?;
+                    }
+                }
             }
 
             if node.first_child().is_some() {
